﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

namespace FlacPlugIn.Banners {
  public class Persist<T> where T: Persistable<T>, new() {
    private string _filename = null;
    private T _container = null;

    private Persist() { }

    public Persist(string filename) {
      Load(filename);
    }

    public string FileName {
      get { return _filename; }
      private set { _filename = value; }
    }

    public T Container {
      get { return _container; }
      private set { _container = value; }
    }

    public T Load(string filename) {
      if(filename == FileName) return Container;
      if(!File.Exists(filename)) return CreateContainer(filename);

      XmlTextReader reader = new XmlTextReader(filename);
      try {
        XmlSerializer serializer = new XmlSerializer(typeof(T));
        Container = serializer.Deserialize(reader) as T;
        Container.Parent = this;
        FileName = filename;
        return Container;
      } finally {
        reader.Close();
      }
    }

    private T CreateContainer(string filename) {
      Container = new T();
      Container.Parent = this;
      FileName = filename;
      return Container;
    }

    public void Save() {
      lock(this) {
        XmlTextWriter writer = new XmlTextWriter(FileName, Encoding.UTF8);
        try {
          XmlSerializer serializer = new XmlSerializer(typeof(T));
          serializer.Serialize(writer, Container);
        } finally {
          writer.Close();
        }
      }
    }
  }
}
