﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EagleWare.Libraries.GeneralNonsense;
using EagleWare.Libraries.FreelancerTrivia;
using EagleWare.Libraries.FLServerCommunications;
using EagleWare.Libraries.FLServerCommunications.Events;

namespace FlacPlugIn.DynamicEconomy {
  internal class BaseManager {
    public static BaseManager Instance { get { return Singleton<BaseManager>.Instance; } }

    private Dictionary<uint, List<uint>> _linkedBases;
    private Dictionary<uint, List<BaseCommodity>> _serverBases;
    private bool _timerSet = false;

    private BaseManager() {
      Producers = new Producers();
      Consumers = new Consumers();
      Resellers = new Resellers();
      _linkedBases = new Dictionary<uint, List<uint>>();
    }

    public Producers Producers { get; private set; }

    public Consumers Consumers { get; private set; }

    public Resellers Resellers { get; private set; }

    public void InitializeServer() {
      Producers.InitializeServer();
      Consumers.InitializeServer();
      Resellers.InitializeServer();
      SetUpdateTimer();
    }

    public void ProcessTrade(uint baseID, uint goodID, int count, bool removeFromStock) {
      Producers.ProcessTrade(baseID, goodID, count, removeFromStock);
      Consumers.ProcessTrade(baseID, goodID, count, removeFromStock);
      Resellers.ProcessTrade(baseID, goodID, count, removeFromStock);
    }

    private void AddBaseToList(List<uint> list, uint baseID) {
      if(list.Contains(baseID)) return;
      list.Add(baseID);
    }

    public void LinkBase(string base1, string base2) {
      LinkBase(Utilities.Instance.GetHash(base1), Utilities.Instance.GetHash(base2));
    }

    private void LinkBase(uint baseID, uint linkedBaseID) {
      List<uint> bases = new List<uint>();
      if(_linkedBases.ContainsKey(baseID) || _linkedBases.ContainsKey(linkedBaseID)) {
        bases = _linkedBases.ContainsKey(baseID)? _linkedBases[baseID]: _linkedBases[linkedBaseID];
      }
      AddBaseToList(bases, baseID);
      AddBaseToList(bases, linkedBaseID);
      _linkedBases[baseID] = bases;
      _linkedBases[linkedBaseID] = bases;
    }

    public List<uint> GetLinkedBases(uint baseID) {
      return _linkedBases.ContainsKey(baseID)? _linkedBases[baseID]: new List<uint>();
    }

    private void SetUpdateTimer() {
      if(_timerSet) return;
      _timerSet = true;
      Utilities.Instance.TimerFactory.CreateTimer(
        Convert.ToUInt32(Utilities.Instance.GetNextSliceExpiration().Subtract(DateTime.UtcNow).TotalSeconds),
        DoTimeSliceUpdate
      );
    }

    private void DoTimeSliceUpdate() {
      _serverBases = new Dictionary<uint, List<BaseCommodity>>();
      Producers.DoTimeSliceUpdate();
      Consumers.DoTimeSliceUpdate();
      Resellers.DoTimeSliceUpdate();
      UpdateServerCommodities();
    }

    private void UpdateServerCommodities() {
      foreach(uint baseID in _serverBases.Keys) {
        SetBaseCommodity.Fire(baseID, _serverBases[baseID]);
      }
    }

    public void AddServerCommodity(uint baseID, BaseCommodity commodity) {
      List<BaseCommodity> commodities = _serverBases.ContainsKey(baseID)?
        _serverBases[baseID]: new List<BaseCommodity>();

      commodities.Add(commodity);
      _serverBases[baseID] = commodities;
    }
  }
}
