﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EagleWare.Libraries.GeneralNonsense;

namespace FlacPlugIn.DynamicEconomy {
  internal class Producers:Bases {
    private class Producer: Base {
      public Producer(uint baseID): base(baseID) { }

      public void AddProducerCommodity(uint goodID, float nominalPrice, float resellerRate) {
        if(GetCommodity(goodID) != null) return;
        AddCommodity(new ProducerCommodity(BaseID, goodID, nominalPrice, resellerRate));
      }

      public ProducerCommodity GetProducerCommodity(uint goodID) {
        return GetCommodity(goodID) as ProducerCommodity;
      }
    }

    private Logger _log = Utilities.Instance.Log;

    public Producers() { }

    private void AddProducer(uint baseID) {
      AddBase(new Producer(baseID));
    }

    private Producer GetProducer(uint baseID) {
      return GetBase(baseID) as Producer;
    }

    public float GetSupplierPrice(uint goodID) {
      return BaseCollection.Min(
        delegate(Base thisBase) {
          ProducerCommodity commodity = (thisBase as Producer).GetProducerCommodity(goodID);
          return commodity == null? Single.MaxValue: commodity.ResellerPrice;
        }
      );
    }

    public void AddProducerCommodity(uint baseID, uint goodID, float nominalPrice, float resellerRate) {
      AddProducer(baseID);
      GetProducer(baseID).AddProducerCommodity(goodID, nominalPrice, resellerRate);
    }

    private Producer GetSupplier(uint goodID) {
      return BaseCollection.First(
        delegate(Base thisBase) {
          ProducerCommodity commodity = (thisBase as Producer).GetProducerCommodity(goodID);
          return commodity != null && commodity.ResellerPrice == GetSupplierPrice(goodID);
        }
      ) as Producer;
    }

    public float SetGlobalResellerRate(uint goodID, float resellerRate) {
      if(BaseCollection.Count == 0) return resellerRate;
      float maximumRate = BaseCollection.Max(
        delegate(Base thisBase) {
          ProducerCommodity commodity = (thisBase as Producer).GetProducerCommodity(goodID);
          return commodity == null? Single.MinValue: commodity.ResellerRate;
        }
      );
      if(resellerRate <= maximumRate) return maximumRate;
      foreach(Base thisBase in BaseCollection) {
        ProducerCommodity commodity = (thisBase as Producer).GetProducerCommodity(goodID);
        if(commodity == null) continue;
        commodity.ResellerRate = resellerRate;
      }
      return resellerRate;
    }

    public int GetSupplierStock(uint goodID) {
      return GetSupplier(goodID).GetProducerCommodity(goodID).InStock;
    }

    public uint GetSupplierBase(uint goodID) {
      return GetSupplier(goodID).BaseID;
    }
  }
}
