﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using System.Reflection;
using EagleWare.Libraries.GeneralNonsense;
using EagleWare.Libraries.FreelancerTrivia;
using ETM=EagleWare.Libraries.GeneralNonsense.Timers;
using ETH=EagleWare.Libraries.GeneralNonsense.Threading;

namespace FlacPlugIn.DynamicEconomy {
  internal class Utilities {
    public static Utilities Instance { get { return Singleton<Utilities>.Instance; } }

    private DateTime _nextSliceExpiration = DateTime.UtcNow;
    private Dictionary<uint, string> _hashes;
    private XmlDocument _configDocument = null;

    private Utilities() {
      _hashes = new Dictionary<uint, string>();
    }

    public int NominalCapacity { get { return 3000; } }

    public Logger Log { get; set; }

    public ETM.Factory TimerFactory { get; set; }

    public ETH.Factory ThreadingFactory { get; set; }

    public float GetPriceFactor(int currentCapacity) {
      float value = currentCapacity;
      value = 1 + (value/NominalCapacity);
      return Convert.ToSingle(1 + (2/Math.Pow(value, 3)));
    }

    public DateTime GetNextSliceExpiration() {
      while(_nextSliceExpiration < DateTime.UtcNow.AddMinutes(1)) {
        _nextSliceExpiration = _nextSliceExpiration.AddMinutes(7);
      }
      return _nextSliceExpiration;
    }

    public uint GetHash(string value) {
      AddDescription(value);
      return Hash.Instance.Calculate(value);
    }

    public void AddDescription(string value) {
      uint hash = Hash.Instance.Calculate(value);
      if(_hashes.ContainsKey(hash)) return;
      _hashes.Add(hash, value);
    }

    public string GetDescription(uint hash) {
      return _hashes.ContainsKey(hash)? _hashes[hash]: hash.ToString();
    }

    public XmlElement ConfigurationNode {
      get {
        if(_configDocument == null) {
          _configDocument = new XmlDocument();
          _configDocument.Load(Path.Combine(
            Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location), "DynamicEconomyConfig.xml"
          ));
        }
        return _configDocument.DocumentElement;
      }
    }
  }
}
