﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using EagleWare.Libraries.GeneralNonsense;
using EagleWare.Libraries.FlacCommands;
using EagleWare.Libraries.FLServerCommunications;
using EagleWare.Libraries.FLServerCommunications.Events;
using EagleWare.Libraries.FLServerCommunications.Notifications;

namespace FlacPlugIn.BattleCry {
  class Cry: PlayerCommand {
      private Dictionary<string, Action<string>> _messageAction;
      private Regex _shoutRegularExpression;
      private Logger _log = Settings.Instance.Logger;
      internal Player cryplayer = null;
      internal string cryconfigfile = null;
      internal string crytext = null;


      private void InitializeActions()
      {          
          _messageAction = new Dictionary<string, Action<string>>();
          _messageAction["cry"] = SendCry;
          _messageAction["set"] = SetCry;
          _messageAction["remove"] = RemoveCry;
      }

      public void GetPlayerFolder()
      {
          string playerpath = cryplayer.SaveGameFullPath;
          string playerdir;          
          string twpath4;          
          System.IO.FileInfo sourceFileInfo = new System.IO.FileInfo(playerpath);
          playerdir = sourceFileInfo.DirectoryName;
          twpath4 = sourceFileInfo.Name;
          cryconfigfile = playerdir + "/cry[" + cryplayer.Name + " - " + twpath4 + "].txt";
      }

      private void GetPlayerConfig()
      {
          GetPlayerFolder();
          TextReader tr = new StreamReader(cryconfigfile);
          if (File.Exists(cryconfigfile))
          {
              crytext = tr.ReadLine();         
          }
          else
          {
              StreamWriter CreateFile = File.CreateText(cryconfigfile);
              CreateFile.Close();
              crytext = tr.ReadLine(); 
          }         
                   
          tr.Close();
      }

    public Cry() {
        InitializeActions();
        string options = string.Join("|", new List<string>(_messageAction.Keys).ToArray());
        string pattern = string.Format(@"^\s*(?<Option>((?i){0}(?-i)))\s+(?<Message>.+)$", options);
        _shoutRegularExpression = new Regex(pattern, RegexOptions.Singleline | RegexOptions.CultureInvariant);
      Syntax += " cry|set|remove";
      Description = "Sends out the set BattleCry to the Current System.";
    }

    /*private void SendCry(Player user, string parameters)
    {
        GetSystem.Fire(user, delegate(SystemUpdate update)
        {
            foreach (Player player in Settings.Instance.ServerInformation.OnlinePlayers)
            {
                SendSystemChat(update, player, user, parameters);
            }
        });
    }*/

    private void SendCry(string message)
    {
        GetSystem.Fire(cryplayer, delegate(SystemUpdate update)
        {
            foreach (Player player in Settings.Instance.ServerInformation.OnlinePlayers)
            {
                SendSystemChat(update, cryplayer, player, crytext);
            }
        });
    }

    private void SetCry(string message)
    {        
        PrivateChat.Fire(cryplayer, string.Format("The cry - {1} - has been set.", message));
    }

    private void RemoveCry(string message)
    {      
        PrivateChat.Fire(cryplayer, string.Format("The cry - {1} - has been removed.", message));
    }

    public override void Execute(Player user, string parameters) {
       GetPlayerConfig();
        
        if (!_shoutRegularExpression.IsMatch(parameters))
        {
            // Invalid input, show error message to user
            PrivateChat.Fire(
       user,
       string.Format("Incorrect parameter '{0}', use either send|set|remove")
     );
            return;
        }

        cryplayer = user;

        Match match = _shoutRegularExpression.Match(parameters);

        _messageAction[match.Groups["Option"].Value.ToLower()](match.Groups["Message"].Value);

        /*
        switch (match.Groups["Option"].Value.ToLower())
        {
            case "cry":
                SendCry(parameters);
                PrivateChat.Fire(user, "battleCry Sent!");
                break;
            case "set":
                SetCry(parameters);
                PrivateChat.Fire(user, String.Format("The message - {0} - has been set", parameters));
                break;
            case "remove":
                RemoveCry(null);
                PrivateChat.Fire(user, "BattleCry has been Removed");
                break;
            default:
                
                break;
        }*/
        
    
    }

    private void SendSystemChat(SystemUpdate targetSystem, Player to, Player from, string message) {
      GetSystem.Fire(to, delegate(SystemUpdate update) {
        if(targetSystem.System != update.System) return;
        _log.Info("Sending message to {0}", to.Name);
        PrivateChat.Fire(to, string.Format("{0}: {1}", from.Name, message));
      });
    }
  }
}
