﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using EagleWare.Libraries.FlacCommands;
using EagleWare.Libraries.FLServerCommunications;
using EagleWare.Libraries.FLServerCommunications.Events;

namespace FlacPlugIn.ShoutPlug
{
    internal class Shout : AdminCommand
    {
        private Dictionary<string, uint> _sizes;
        private Dictionary<string, uint> _colours;        
        private Regex _shoutRegularExpression;
        //public string colour = "red";
       // public string style;
        //public string format;

        private void InitializeSizes()
        {
            _sizes = new Dictionary<string, uint>();
            _sizes["small"] = 0x50;
            _sizes["medium"] = 0x58;
            _sizes["large"] = 0x10;
            _sizes["huge"] = 0x20;
        }

        private void InitializeColours()
        {
            _colours = new Dictionary<string, uint>();
            _colours["red"] = 0x0000FF00;
            _colours["green"] = 0x00FF0000;
            _colours["blue"] = 0xFF800000;
            _colours["yellow"] = 0x00FFFF00;
            _colours["orange"] = 0x0080FF00;
            _colours["white"] = 0xFFFFFF00;
        }

      
        public Shout()
        {

            InitializeSizes();
            InitializeColours();

            string options = string.Join("|", new List<string>(_sizes.Keys).ToArray());
            string colour = string.Join("|", new List<string>(_colours.Keys).ToArray());

            string pattern = string.Format(@"^\s*(?<Size>((?i){0}(?-i)))\s+(?<Colour>((?i){1}(?-i)))\s+(?<Message>.+)$", options, colour);
            //string pattern = string.Format(@"^\s*(?<Size>((?i){0}(?-i)))\s+(?<Colour>((?i){1}(?-i)))s+(?<Message>.+)$i", options, colour);
           // string pattern = string.Format(@"^\s*(?<Size>((?i){0}(?-i)))\s+(?<Colour>((?i){1}(?-i)))\s+(?<Message>.+)$i", options, colour);
            _shoutRegularExpression = new Regex(pattern, RegexOptions.Singleline | RegexOptions.CultureInvariant | RegexOptions.IgnoreCase);
            Syntax += string.Format(" [{0}] [{1}] [message]", options, colour);
            Description = "Shouts to universe.";
        }

        public override void Execute(Player user, string parameters)
        {
            if (!_shoutRegularExpression.IsMatch(parameters))
            {
                // Invalid input, show error message to user
                PrivateChat.Fire(user, string.Format("Incorrect parameters")
         );
                return;
            }
            Match match = _shoutRegularExpression.Match(parameters);
            UniverseCustomChat.Fire(
              _colours[match.Groups["Colour"].Value.ToLower()] | _sizes[match.Groups["Size"].Value.ToLower()],
              match.Groups["Message"].Value
            );
        }

    }
}
